@@ -18,6 +18,8 @@ import os
 
 from configparser import NoOptionError
 import numpy as np
+from tensorflow.keras.preprocessing.image import ImageDataGenerator
+from snntoolbox.utils.utils import import_helpers
 
 
 def get_dataset(config):
@@ -45,13 +47,17 @@ def get_dataset(config):
 
     """
 
-    testset = None
+    testset = {}
     normset = try_get_normset_from_scalefacs(config)
     dataset_path = config.get('paths', 'dataset_path')
+    dataset_format = config.get('input', 'dataset_format')
+    normalize_thresholds = config.getboolean('loihi', 'normalize_thresholds',
+                                             fallback=False)
     is_testset_needed = config.getboolean('tools', 'evaluate_ann') or \
-        config.getboolean('tools', 'simulate')
-    is_normset_needed = config.getboolean('tools', 'normalize') and \
-        normset is None
+        config.getboolean('tools', 'simulate') or normalize_thresholds
+    is_normset_needed = normalize_thresholds or (
+            config.getboolean('tools', 'normalize') and normset == {})
+    batch_size = config.getint('simulation', 'batch_size')
 
     # _______________________________ Keras __________________________________#
     try:
@@ -66,28 +72,26 @@ def get_dataset(config):
                 'x_test': x_test[:num_to_test],
                 'y_test': y_test[:num_to_test]}
             if is_normset_needed:
-                normset = {'x_norm': x_test}
+                normset['x_norm'] = x_test
             return normset, testset
     except (NoOptionError, ImportError) as e:
         print("Warning:", e)
 
     # ________________________________ npz ___________________________________#
-    if config.get('input', 'dataset_format') == 'npz':
+    if dataset_format == 'npz':
         print("Loading data set from '.npz' files in {}.\n".format(
             dataset_path))
         if is_testset_needed:
             num_to_test = config.getint('simulation', 'num_to_test')
-            testset = {
-                'x_test': load_npz(dataset_path, 'x_test.npz')[:num_to_test],
-                'y_test': load_npz(dataset_path, 'y_test.npz')[:num_to_test]}
-            assert testset, "Test set empty."
+            x_test = load_npz(dataset_path, 'x_test.npz')[:num_to_test]
+            y_test = load_npz(dataset_path, 'y_test.npz')[:num_to_test]
+            testset = {'x_test': x_test, 'y_test': y_test}
         if is_normset_needed:
-            normset = {'x_norm': load_npz(dataset_path, 'x_norm.npz')}
-            assert normset, "Normalization set empty."
+            x_norm = load_npz(dataset_path, 'x_norm.npz')
+            normset['x_norm'] = x_norm
 
     # ________________________________ jpg ___________________________________#
-    elif config.get('input', 'dataset_format') in {'jpg', 'png'}:
-        from tensorflow.keras.preprocessing.image import ImageDataGenerator
+    elif dataset_format in {'jpg', 'png'}:
         print("Loading data set from ImageDataGenerator, using images in "
               "{}.\n".format(dataset_path))
         # Transform str to dict
@@ -103,7 +107,6 @@ def get_dataset(config):
 
         # Get proprocessing function
         if 'preprocessing_function' in datagen_kwargs:
-            from snntoolbox.utils.utils import import_helpers
             helpers = import_helpers(datagen_kwargs['preprocessing_function'],
                                      config)
             datagen_kwargs['preprocessing_function'] = \
@@ -111,33 +114,36 @@ def get_dataset(config):
 
         dataflow_kwargs['directory'] = dataset_path
         if 'batch_size' not in dataflow_kwargs:
-            dataflow_kwargs['batch_size'] = config.getint('simulation',
-                                                          'batch_size')
+            dataflow_kwargs['batch_size'] = batch_size
         datagen = ImageDataGenerator(**datagen_kwargs)
-        # Compute quantities required for featurewise normalization
-        # (std, mean, and principal components if ZCA whitening is applied)
-        rs = datagen_kwargs['rescale'] if 'rescale' in datagen_kwargs else None
-        x_orig = ImageDataGenerator(rescale=rs).flow_from_directory(
-            **dataflow_kwargs).next()[0]
-        datagen.fit(x_orig)
+        if (datagen.featurewise_center or datagen.featurewise_std_normalization
+                or datagen.zca_whitening):
+            # Compute quantities required for featurewise normalization
+            # (std, mean, and principal components if ZCA whitening is applied)
+            rs = datagen_kwargs.get('rescale', None)
+            x_orig = ImageDataGenerator(rescale=rs).flow_from_directory(
+                **dataflow_kwargs).next()[0]
+            datagen.fit(x_orig)
         if is_normset_needed:
             shuffle = dataflow_kwargs.get('shuffle')
             dataflow_kwargs['shuffle'] = True
-            normset = {
-                'dataflow': datagen.flow_from_directory(**dataflow_kwargs)}
+            normset['dataflow'] = \
+                datagen.flow_from_directory(**dataflow_kwargs)
             dataflow_kwargs['shuffle'] = shuffle
-            assert normset, "Normalization set empty."
         if is_testset_needed:
             testset = {
                 'dataflow': datagen.flow_from_directory(**dataflow_kwargs)}
-            assert testset, "Test set empty."
 
     # _______________________________ aedat __________________________________#
-    elif config.get('input', 'dataset_format') == 'aedat':
+    elif dataset_format == 'aedat':
         if is_normset_needed:
-            normset = {'x_norm': load_npz(dataset_path, 'x_norm.npz')}
-            assert normset, "Normalization set empty."
-        testset = {}
+            print("Loading normalization dataset from '.npz' file in {}.\n"
+                  "".format(dataset_path))
+            x_norm = load_npz(dataset_path, 'x_norm.npz')
+            normset['x_norm'] = x_norm
+            # For Loihi threshold normalization we need to pass the
+            # normalization data in the testset dict.
+            testset = {'x_norm': x_norm}
 
     return normset, testset
 
@@ -159,14 +165,13 @@ def try_get_normset_from_scalefacs(config):
     : Union[dict, None]
         A dictionary with single key 'scale_facs'. The corresponding value is
         itself a dictionary containing the scale factors for each layer.
-        Returns ``None`` if no scale factors were found.
+        Returns empty set if no scale factors were found.
     """
 
     newpath = os.path.join(config.get('paths', 'log_dir_of_current_run'),
                            'normalization')
     if not os.path.exists(newpath):
         os.makedirs(newpath)
-        return
     filepath = os.path.join(newpath, config.get('normalization',
                                                 'percentile') + '.json')
     if os.path.isfile(filepath):
@@ -174,6 +179,8 @@ def try_get_normset_from_scalefacs(config):
         with open(filepath) as f:
             return {'scale_facs': json.load(f)}
 
+    return {}
+
 
 def to_categorical(y, nb_classes):
     """Convert class vector to binary class matrix.
