@@ -97,6 +97,9 @@ class ModelParser(AbstractModelParser):
             attributes['parameters'] = tuple(attributes['parameters'])
             attributes['use_bias'] = True
 
+    def parse_transpose_convolution(self, layer, attributes):
+        self.parse_convolution(layer, attributes)
+
     def parse_pooling(self, layer, attributes):
         pass
 
@@ -111,6 +114,17 @@ class ModelParser(AbstractModelParser):
     def parse_concatenate(self, layer, attributes):
         pass
 
+    @property
+    def input_layer_name(self):
+        # Check if model has a dedicated input layer. If so, return its name.
+        # Otherwise, the first layer might be a conv layer, so we return
+        # 'input'.
+        first_layer = self.input_model.layers[0]
+        if 'Input' in self.get_type(first_layer):
+            return first_layer.name
+        else:
+            return 'input'
+
 
 def load(path, filename, **kwargs):
     """Load network from file.
@@ -202,13 +216,11 @@ def evaluate(val_fn, batch_size, num_to_test, x_test=None, y_test=None,
 
     if x_test is not None:
         score = val_fn(x_test, y_test, batch_size, verbose=0)
+    elif dataflow is not None:
+        steps = len(dataflow)
+        score = val_fn(dataflow, batch_size=batch_size, verbose=0, steps=steps)
     else:
-        score = np.zeros(3)
-        batches = int(num_to_test / batch_size)
-        for i in range(batches):
-            x_batch, y_batch = dataflow.next()
-            score += val_fn(x_batch, y_batch, batch_size, verbose=0)
-        score /= batches
+        raise NotImplementedError
 
     print("Top-1 accuracy: {:.2%}".format(score[1]))
     print("Top-5 accuracy: {:.2%}\n".format(score[2]))
