@@ -40,6 +40,8 @@ from abc import abstractmethod
 from tensorflow import keras
 import numpy as np
 
+IS_CHANNELS_FIRST = keras.backend.image_data_format() == 'channels_first'
+
 
 class AbstractModelParser:
     """Abstract base class for neural network model parsers.
@@ -162,26 +164,64 @@ class AbstractModelParser:
             if layer_type == 'GlobalAveragePooling2D':
                 print("Replacing GlobalAveragePooling by AveragePooling "
                       "plus Flatten.")
-                a = 1 if keras.backend.image_data_format() == 'channels_last' \
-                    else 2
+                _layer_type = 'AveragePooling2D'
+                axis = 2 if IS_CHANNELS_FIRST else 1
                 self._layer_list.append(
-                    {'layer_type': 'AveragePooling2D',
-                     'name': self.get_name(layer, idx, 'AveragePooling2D'),
-                     'pool_size': (layer.input_shape[a: a + 2]),
+                    {'layer_type': _layer_type,
+                     'name': self.get_name(layer, idx, _layer_type),
+                     'pool_size': (layer.input_shape[axis: axis + 2]),
                      'inbound': self.get_inbound_names(layer, name_map),
                      'strides': [1, 1]})
-                name_map['AveragePooling2D' + str(idx)] = idx
+                name_map[_layer_type + str(idx)] = idx
                 idx += 1
-                num_str = str(idx) if idx > 9 else '0' + str(idx)
-                shape_string = str(np.prod(layer.output_shape[1:]))
+                _layer_type = 'Flatten'
+                num_str = self.format_layer_idx(idx)
+                shape_str = str(np.prod(layer.output_shape[1:]))
                 self._layer_list.append(
-                    {'name': num_str + 'Flatten_' + shape_string,
-                     'layer_type': 'Flatten',
+                    {'name': num_str + _layer_type + '_' + shape_str,
+                     'layer_type': _layer_type,
                      'inbound': [self._layer_list[-1]['name']]})
-                name_map['Flatten' + str(idx)] = idx
+                name_map[_layer_type + str(idx)] = idx
                 idx += 1
                 inserted_flatten = True
 
+            if layer_type == 'Add':
+                print("Replacing Add layer by Concatenate plus Conv.")
+                shape = layer.output_shape
+                if IS_CHANNELS_FIRST:
+                    axis = 1
+                    c, h, w = shape[1:]
+                    shape_str = '{}x{}x{}'.format(2 * c, h, w)
+                else:
+                    axis = -1
+                    h, w, c = shape[1:]
+                    shape_str = '{}x{}x{}'.format(h, w, 2 * c)
+                _layer_type = 'Concatenate'
+                num_str = self.format_layer_idx(idx)
+                self._layer_list.append({
+                    'layer_type': _layer_type,
+                    'name': num_str + _layer_type + '_' + shape_str,
+                    'inbound': self.get_inbound_names(layer, name_map),
+                    'axis': axis})
+                name_map[_layer_type + str(idx)] = idx
+                idx += 1
+                _layer_type = 'Conv2D'
+                num_str = self.format_layer_idx(idx)
+                shape_str = '{}x{}x{}'.format(*shape[1:])
+                weights = np.zeros([1, 1, 2 * c, c])
+                for k in range(c):
+                    weights[:, :, k::c, k] = 1
+                self._layer_list.append({
+                    'name': num_str + _layer_type + '_' + shape_str,
+                    'layer_type': _layer_type,
+                    'inbound': [self._layer_list[-1]['name']],
+                    'filters': c,
+                    'activation': 'relu',  # Default nonlinearity of SNN
+                    'parameters': (weights, np.zeros(c)),
+                    'kernel_size': 1})
+                name_map[str(id(layer))] = idx
+                idx += 1
+
             if layer_type not in snn_layers:
                 print("Skipping layer {}.".format(layer_type))
                 continue
@@ -198,6 +238,7 @@ class AbstractModelParser:
                 print("Replacing max by average pooling.")
                 layer_type = 'AveragePooling2D'
 
+            # If we inserted a layer, need to set the right inbound layer here.
             if inserted_flatten:
                 inbound = [self._layer_list[-1]['name']]
                 inserted_flatten = False
@@ -228,6 +269,9 @@ class AbstractModelParser:
             if layer_type == 'SparseDepthwiseConv2D':
                 self.parse_sparse_depthwiseconvolution(layer, attributes)
 
+            if layer_type == 'Conv2DTranspose':
+                self.parse_transpose_convolution(layer, attributes)
+
             if layer_type in ['Sparse', 'SparseConv2D',
                               'SparseDepthwiseConv2D']:
                 weights, bias, mask = attributes['parameters']
@@ -239,7 +283,8 @@ class AbstractModelParser:
 
                 self.absorb_activation(layer, attributes)
 
-            if layer_type in {'Dense', 'Conv1D', 'Conv2D', 'DepthwiseConv2D'}:
+            if layer_type in {'Dense', 'Conv1D', 'Conv2D', 'DepthwiseConv2D',
+                              'Conv2DTranspose'}:
                 weights, bias = attributes['parameters']
 
                 weights, bias = modify_parameter_precision(
@@ -361,7 +406,7 @@ class AbstractModelParser:
                     break
         if len(self._layer_list) == 0 or \
                 any([self.get_type(inb) == 'InputLayer' for inb in inbound]):
-            return ['input']
+            return [self.input_layer_name]
         else:
             inb_idxs = [name_map[str(id(inb))] for inb in inbound]
             return [self._layer_list[i]['name'] for i in inb_idxs]
@@ -479,10 +524,32 @@ class AbstractModelParser:
         shape_string[-1] = shape_string[-1][:-1]
         shape_string = "".join(shape_string)
 
-        num_str = str(idx) if idx > 9 else '0' + str(idx)
+        num_str = self.format_layer_idx(idx)
 
         return num_str + layer_type + shape_string
 
+    def format_layer_idx(self, idx):
+        """Pad the layer index with the appropriate amount of zeros.
+
+        The number of zeros used for padding is determined by the maximum index
+        (i.e. the number of layers in the network).
+
+        Parameters
+        ----------
+
+        idx: int
+            Layer index.
+
+        Returns
+        -------
+
+        num_str: str
+            Zero-padded layer index.
+        """
+
+        max_idx = len(self.input_model.layers)
+        return str(idx).zfill(len(str(max_idx)))
+
     @abstractmethod
     def get_output_shape(self, layer):
         """Get output shape of a ``layer``.
@@ -512,7 +579,7 @@ class AbstractModelParser:
             assert len(previous_layers) == 1, \
                 "Layer to flatten must be unique."
             print("Inserting layer Flatten.")
-            num_str = str(idx) if idx > 9 else '0' + str(idx)
+            num_str = self.format_layer_idx(idx)
             shape_string = str(np.prod(prev_layer_output_shape[1:]))
             self._layer_list.append({
                 'name': num_str + 'Flatten_' + shape_string,
@@ -574,6 +641,9 @@ class AbstractModelParser:
     def parse_sparse_convolution(self, layer, attributes):
         pass
 
+    def parse_transpose_convolution(self, layer, attributes):
+        pass
+
     def parse_sparse_depthwiseconvolution(self, layer, attributes):
         pass
 
@@ -723,8 +793,9 @@ class AbstractModelParser:
         """
 
         img_input = keras.layers.Input(
-            batch_shape=self.get_batch_input_shape(), name='input')
-        parsed_layers = {'input': img_input}
+            batch_shape=self.get_batch_input_shape(),
+            name=self.input_layer_name)
+        parsed_layers = {self.input_layer_name: img_input}
         print("Building parsed model...\n")
         for layer in self._layer_list:
             # Replace 'parameters' key with Keras key 'weights'
@@ -749,9 +820,10 @@ class AbstractModelParser:
         self.parsed_model = keras.models.Model(img_input, parsed_layers[
             self._layer_list[-1]['name']])
         # Optimizer and loss do not matter because we only do inference.
-        self.parsed_model.compile(
-            'sgd', 'categorical_crossentropy',
-            ['accuracy', keras.metrics.top_k_categorical_accuracy])
+        top_k = keras.metrics.TopKCategoricalAccuracy(
+            self.config.getint('simulation', 'top_k'))
+        self.parsed_model.compile('sgd', 'categorical_crossentropy',
+                                  ['accuracy', top_k])
         # Todo: Enable adding custom metric via self.input_model.metrics.
         self.parsed_model.summary()
         return self.parsed_model
@@ -794,6 +866,10 @@ class AbstractModelParser:
 
         return score
 
+    @property
+    def input_layer_name(self):
+        return 'input'
+
 
 def absorb_bn_parameters(weight, bias, mean, var_eps_sqrt_inv, gamma, beta,
                          axis, image_data_format, is_depthwise=False):
@@ -811,7 +887,7 @@ def absorb_bn_parameters(weight, bias, mean, var_eps_sqrt_inv, gamma, beta,
     # [height, width, num_input_channels, num_output_channels],
     # and layers like [batch_size, channels, height, width] or
     # [batch_size, height, width, channels].
-    if weight.ndim == 4:
+    if weight.ndim == 4:  # Conv2D
 
         channel_axis = 2 if is_depthwise else 3
 
@@ -821,6 +897,16 @@ def absorb_bn_parameters(weight, bias, mean, var_eps_sqrt_inv, gamma, beta,
             layer2kernel_axes_map = [None, 0, 1, channel_axis]
 
         axis = layer2kernel_axes_map[axis]
+    elif weight.ndim == 3:  # Conv1D
+
+        channel_axis = 2
+
+        if image_data_format == 'channels_first':
+            layer2kernel_axes_map = [None, channel_axis, 0]
+        else:
+            layer2kernel_axes_map = [None, 0, channel_axis]
+
+        axis = layer2kernel_axes_map[axis]
 
     broadcast_shape = [1] * weight.ndim
     broadcast_shape[axis] = weight.shape[axis]
@@ -1098,7 +1184,7 @@ def get_fanin(layer):
 
     layer_type = get_type(layer)
     if 'Conv' in layer_type:
-        ax = 1 if keras.backend.image_data_format() == 'channels_first' else -1
+        ax = 1 if IS_CHANNELS_FIRST else -1
         fanin = np.prod(layer.kernel_size) * layer.input_shape[ax]
     elif 'Dense' in layer_type:
         fanin = layer.input_shape[1]
@@ -1141,7 +1227,7 @@ def get_fanout(layer, config):
     for next_layer in next_layers:
         if 'Conv' in next_layer.name and not has_stride_unity(next_layer):
             shape = layer.output_shape
-            if 'input' in layer.name:
+            if 'Input' in get_type(layer):
                 shape = fix_input_layer_shape(shape)
             fanout = np.zeros(shape[1:])
             break
@@ -1178,11 +1264,60 @@ def get_fanout_array(layer_pre, layer_post, is_depthwise_conv=False):
     the post-synaptic layer has stride > 1, the fan-out varies between neurons.
     """
 
-    ax = 1 if keras.backend.image_data_format() == 'channels_first' else 0
+    shape = layer_pre.output_shape
+    if 'Input' in get_type(layer_pre):
+        shape = fix_input_layer_shape(shape)
+    ndim = len(shape)
+
+    if ndim == 4:
+        return _get_fanout_array_2D(layer_pre, layer_post, is_depthwise_conv)
+    elif ndim == 3:
+        return _get_fanout_array_1D(layer_pre, layer_post, is_depthwise_conv)
+    else:
+        raise NotImplementedError
+
+
+def _get_fanout_array_1D(layer_pre, layer_post, is_depthwise_conv=False):
+    ax = 1 if IS_CHANNELS_FIRST else 0
+
+    ny = layer_post.output_shape[1 + ax]  # Height of feature map
+    nz = layer_post.output_shape[ax if ax else -1]  # Number of channels
+    ky = layer_post.kernel_size[0]  # Height of kernel
+    py = int((ky - 1) / 2) if layer_post.padding == 'same' else 0
+    sy = layer_post.strides[0]
+
+    shape = layer_pre.output_shape
+    if 'Input' in get_type(layer_pre):
+        shape = fix_input_layer_shape(shape)
+    fanout = np.zeros(shape[1:])
+
+    for y_pre in range(fanout.shape[0 + ax]):
+        y_post = [int((y_pre + py) / sy)]
+        wy = (y_pre + py) % sy
+        i = 1
+        while wy + i * sy < ky:
+            y = y_post[0] - i
+            if 0 <= y < ny:
+                y_post.append(y)
+            i += 1
+
+        if ax:
+            fanout[:, y_pre] = len(y_post)
+        else:
+            fanout[y_pre, :] = len(y_post)
+
+    if not is_depthwise_conv:
+        fanout *= nz
+
+    return fanout
+
+
+def _get_fanout_array_2D(layer_pre, layer_post, is_depthwise_conv=False):
+    ax = 1 if IS_CHANNELS_FIRST else 0
 
     nx = layer_post.output_shape[2 + ax]  # Width of feature map
     ny = layer_post.output_shape[1 + ax]  # Height of feature map
-    nz = layer_post.output_shape[ax]  # Number of channels
+    nz = layer_post.output_shape[ax if ax else -1]  # Number of channels
     kx, ky = layer_post.kernel_size  # Width and height of kernel
     px = int((kx - 1) / 2) if layer_post.padding == 'same' else 0
     py = int((ky - 1) / 2) if layer_post.padding == 'same' else 0
@@ -1190,7 +1325,7 @@ def get_fanout_array(layer_pre, layer_post, is_depthwise_conv=False):
     sy = layer_post.strides[0]
 
     shape = layer_pre.output_shape
-    if 'input' in layer_pre.name:
+    if 'Input' in get_type(layer_pre):
         shape = fix_input_layer_shape(shape)
     fanout = np.zeros(shape[1:])
 
