@@ -18,7 +18,8 @@ import json
 import tensorflow as tf
 from tensorflow.keras.layers import Dense, Flatten, AveragePooling2D, \
     MaxPooling2D, Conv2D, DepthwiseConv2D, ZeroPadding2D, Reshape, Layer, \
-    Concatenate
+    Concatenate, Conv1D, Conv2DTranspose, UpSampling2D
+from tensorflow.python.keras.utils import conv_utils
 
 from snntoolbox.parsing.utils import get_inbound_layers
 
@@ -32,6 +33,14 @@ class SpikeLayer(Layer):
 
     def __init__(self, **kwargs):
         self.config = kwargs.pop(str('config'), None)
+        if self.config is None:
+            from snntoolbox.bin.utils import load_config
+            # Todo: Enable loading config here. Needed when trying to load a
+            #       converted SNN from disk. For now we specify a dummy path.
+            try:
+                self.config = load_config('wdir/log/gui/test/.config')
+            except FileNotFoundError:
+                raise NotImplementedError
         self.layer_type = self.class_name
         self.dt = self.config.getfloat('simulation', 'dt')
         self.duration = self.config.getint('simulation', 'duration')
@@ -40,6 +49,7 @@ class SpikeLayer(Layer):
         self.v_thresh = None
         self.time = None
         self.mem = self.spiketrain = self.impulse = self.spikecounts = None
+        self.mem_input = None  # Used in MaxPooling layers
         self.refrac_until = self.max_spikerate = None
         if clamp_var:
             self.spikerate = self.var = None
@@ -240,8 +250,8 @@ class SpikeLayer(Layer):
                 self.activation_str == 'softmax'):
             # Turn off reset (uncomment second line) to get a faster and better
             # top-1 error. The top-5 error is better when resetting:
-            new = tf.where(tf.not_equal(spikes, 0), tf.zeros_like(mem), mem)
-            # new = tf.identity(mem)
+            # new = tf.where(tf.not_equal(spikes, 0), tf.zeros_like(mem), mem)
+            new = tf.identity(mem)
         elif self.config.get('cell', 'reset') == 'Reset by subtraction':
             if self.payloads:  # Experimental.
                 new = tf.where(tf.not_equal(spikes, 0),
@@ -351,19 +361,21 @@ class SpikeLayer(Layer):
             self.mem.assign(self.init_membrane_potential())
             self.time.assign(self.dt)
         if self.tau_refrac > 0:
-            self.refrac_until.assign(tf.zeros(self.output_shape, self._floatx))
+            self.refrac_until.assign(tf.zeros_like(self.refrac_until))
         if self.spiketrain is not None:
-            self.spiketrain.assign(tf.zeros(self.output_shape, self._floatx))
+            self.spiketrain.assign(tf.zeros_like(self.spiketrain))
         if self.payloads:
-            self.payloads.assign(tf.zeros(self.output_shape, self._floatx))
-            self.payloads_sum.assign(tf.zeros(self.output_shape, self._floatx))
+            self.payloads.assign(tf.zeros_like(self.payloads))
+            self.payloads_sum.assign(tf.zeros_like(self.payloads_sum))
         if self.online_normalization and do_reset:
-            self.spikecounts.assign(tf.zeros(self.output_shape, self._floatx))
+            self.spikecounts.assign(tf.zeros_like(self.spikecounts))
             self.max_spikerate.assign(0)
             self.v_thresh.assign(self._v_thresh)
         if clamp_var and do_reset:
-            self.spikerate.assign(tf.zeros(self.input_shape, self._floatx))
-            self.var.assign(tf.zeros(self.input_shape, self._floatx))
+            self.spikerate.assign(tf.zeros_like(self.spikerate))
+            self.var.assign(tf.zeros_like(self.var))
+        if self.mem_input is not None:
+            self.mem_input.assign(tf.zeros_like(self.mem_input))
 
     @tf.function
     def init_neurons(self, input_shape):
@@ -381,7 +393,7 @@ class SpikeLayer(Layer):
         if self.time is None:
             self.time = tf.Variable(self.dt, name='dt', trainable=False)
         # To save memory and computations, allocate only where needed:
-        if self.tau_refrac > 0 and self.tau_refrac_until is None:
+        if self.tau_refrac > 0 and self.refrac_until is None:
             self.refrac_until = tf.Variable(
                 tf.zeros(output_shape), name='refrac_until', trainable=False)
         if any({'spiketrains', 'spikerates', 'correlation', 'spikecounts',
@@ -506,7 +518,7 @@ class SpikeConcatenate(Concatenate):
     """Spike merge layer"""
 
     def __init__(self, axis, **kwargs):
-        kwargs.pop(str('config'))
+        kwargs.pop(str('config'), None)
         Concatenate.__init__(self, axis, **kwargs)
 
     @staticmethod
@@ -531,7 +543,7 @@ class SpikeFlatten(Flatten):
     """Spike flatten layer."""
 
     def __init__(self, **kwargs):
-        kwargs.pop(str('config'))
+        kwargs.pop(str('config'), None)
         Flatten.__init__(self, **kwargs)
 
     def call(self, x, mask=None):
@@ -560,7 +572,7 @@ class SpikeZeroPadding2D(ZeroPadding2D):
     """Spike ZeroPadding2D layer."""
 
     def __init__(self, **kwargs):
-        kwargs.pop(str('config'))
+        kwargs.pop(str('config'), None)
         ZeroPadding2D.__init__(self, **kwargs)
 
     def call(self, x, mask=None):
@@ -589,7 +601,7 @@ class SpikeReshape(Reshape):
     """Spike Reshape layer."""
 
     def __init__(self, **kwargs):
-        kwargs.pop(str('config'))
+        kwargs.pop(str('config'), None)
         Reshape.__init__(self, **kwargs)
 
     def call(self, x, mask=None):
@@ -639,6 +651,33 @@ class SpikeDense(Dense, SpikeLayer):
         return Dense.call(self, x)
 
 
+class SpikeConv1D(Conv1D, SpikeLayer):
+    """Spike 1D Convolution."""
+
+    def build(self, input_shape):
+        """Creates the layer weights.
+        Must be implemented on all layers that have weights.
+
+        Parameters
+        ----------
+
+        input_shape: Union[list, tuple, Any]
+            Keras tensor (future input to layer) or list/tuple of Keras tensors
+            to reference for weight shape computations.
+        """
+
+        Conv1D.build(self, input_shape)
+        self.init_neurons(input_shape.as_list())
+
+        if self.config.getboolean('cell', 'bias_relaxation'):
+            self.update_b()
+
+    @spike_call
+    def call(self, x, mask=None):
+
+        return Conv1D.call(self, x)
+
+
 class SpikeConv2D(Conv2D, SpikeLayer):
     """Spike 2D Convolution."""
 
@@ -693,6 +732,55 @@ class SpikeDepthwiseConv2D(DepthwiseConv2D, SpikeLayer):
         return DepthwiseConv2D.call(self, x)
 
 
+class SpikeConv2DTranspose(Conv2DTranspose, SpikeLayer):
+    """Spike 2D transpose convolution."""
+
+    def build(self, input_shape):
+        """Creates the layer weights.
+        Must be implemented on all layers that have weights.
+
+        Parameters
+        ----------
+        input_shape: Union[list, tuple, Any]
+            Keras tensor (future input to layer) or list/tuple of Keras tensors
+            to reference for weight shape computations.
+        """
+
+        Conv2DTranspose.build(self, input_shape)
+        self.init_neurons(input_shape.as_list())
+
+        if self.config.getboolean('cell', 'bias_relaxation'):
+            self.update_b()
+
+    @spike_call
+    def call(self, x, mask=None):
+
+        return Conv2DTranspose.call(self, x)
+
+
+class SpikeUpSampling2D(UpSampling2D, SpikeLayer):
+    """Spike upsampling layer."""
+
+    def build(self, input_shape):
+        """Creates the layer weights.
+        Must be implemented on all layers that have weights.
+
+        Parameters
+        ----------
+        input_shape: Union[list, tuple, Any]
+            Keras tensor (future input to layer) or list/tuple of Keras tensors
+            to reference for weight shape computations.
+        """
+
+        UpSampling2D.build(self, input_shape)
+        self.init_neurons(input_shape.as_list())
+
+    @spike_call
+    def call(self, x, mask=None):
+
+        return UpSampling2D.call(self, x)
+
+
 class SpikeAveragePooling2D(AveragePooling2D, SpikeLayer):
     """Spike Average Pooling."""
 
@@ -733,15 +821,25 @@ class SpikeMaxPooling2D(MaxPooling2D, SpikeLayer):
 
         MaxPooling2D.build(self, input_shape)
         self.init_neurons(input_shape.as_list())
+        if self.mem_input is None:
+            self.mem_input = tf.Variable(tf.zeros(input_shape),
+                                         name='mem_input', trainable=False)
 
     @spike_call
     def call(self, x, mask=None):
         """Layer functionality."""
 
-        print("WARNING: Rate-based spiking MaxPooling layer is not "
-              "implemented in TensorFlow backend. Falling back on "
-              "AveragePooling. Switch to Theano backend to use MaxPooling.")
-        return tf.nn.avg_pool2d(x, self.pool_size, self.strides, self.padding)
+        self.mem_input.assign_add(x)
+        _, max_idxs = tf.nn.max_pool_with_argmax(
+            self.mem_input, self.pool_size, self.strides, self.padding.upper(),
+            conv_utils.convert_data_format(self.data_format, 4),
+            include_batch_in_index=True)
+        x_max = tf.scatter_nd(tf.reshape(max_idxs, (-1, 1)),
+                              tf.ones(tf.size(max_idxs)),
+                              [tf.reduce_prod(x.shape)])
+        x_max = tf.reshape(x_max, x.shape)
+        x_masked = x * x_max
+        return MaxPooling2D.call(self, x_masked)
 
 
 custom_layers = {'SpikeFlatten': SpikeFlatten,
@@ -749,7 +847,10 @@ custom_layers = {'SpikeFlatten': SpikeFlatten,
                  'SpikeZeroPadding2D': SpikeZeroPadding2D,
                  'SpikeDense': SpikeDense,
                  'SpikeConv2D': SpikeConv2D,
+                 'SpikeConv1D': SpikeConv1D,
                  'SpikeDepthwiseConv2D': SpikeDepthwiseConv2D,
+                 'SpikeConv2DTranspose': SpikeConv2DTranspose,
                  'SpikeAveragePooling2D': SpikeAveragePooling2D,
                  'SpikeMaxPooling2D': SpikeMaxPooling2D,
-                 'SpikeConcatenate': SpikeConcatenate}
+                 'SpikeConcatenate': SpikeConcatenate,
+                 'SpikeUpSampling2D': SpikeUpSampling2D}
