@@ -11,23 +11,19 @@ simulation duration.
 
 @author: rbodo
 """
+import os
 
-from __future__ import division, absolute_import
-from __future__ import print_function, unicode_literals
-
+import json
 import warnings
 
 import numpy as np
 import theano
-from future import standard_library
 from keras import backend as k
 from keras.layers import Dense, Flatten, AveragePooling2D, MaxPooling2D, Conv2D
 from keras.layers import Layer, Concatenate
 
 from snntoolbox.parsing.utils import get_inbound_layers
 
-standard_library.install_aliases()
-
 # Experimental
 clamp_var = False
 v_clip = False
@@ -52,7 +48,6 @@ class SpikeLayer(Layer):
         if clamp_var:
             self.spikerate = self.var = None
 
-        import os
         from snntoolbox.utils.utils import get_abs_path
         path, filename = \
             get_abs_path(self.config.get('paths', 'filename_clamp_indices'),
@@ -420,8 +415,6 @@ class SpikeLayer(Layer):
             Time step when to stop clamping.
         """
 
-        import json
-
         with open(self.filename_clamp_indices) as f:
             clamp_indices = json.load(f)
 
