@@ -199,12 +199,14 @@ class SNN(AbstractSNN):
         x_flat = np.ravel(data)
         if self._poisson_input:
             self.layers[0].set(rate=list(x_flat / self.rescale_fac * 1000))
-        elif self._dataset_format == 'aedat':
+        elif self._is_aedat_input:
             raise NotImplementedError
         else:
-            spike_times = \
-                [np.linspace(0, self._duration, self._duration * amplitude)
-                 for amplitude in x_flat]
+            spike_times = []
+            for amplitude in x_flat:
+                st = np.linspace(0, self._duration,
+                                 int(self._duration * amplitude))
+                spike_times.append(st)
             self.layers[0].set(spike_times=spike_times)
 
         if is_module_installed('pynn_object_serialisation'):
