@@ -15,7 +15,6 @@ import sys
 from abc import abstractmethod
 import numpy as np
 from tensorflow import keras
-from tensorflow.python.keras.utils.conv_utils import convert_kernel
 
 from snntoolbox.bin.utils import get_log_keys, get_plot_keys, \
     initialize_simulator
@@ -167,7 +166,10 @@ class AbstractSNN:
 
         self.sim = initialize_simulator(config)
 
-        self._dataset_format = self.config.get('input', 'dataset_format')
+        self._is_early_stopping = self.config.getboolean('simulation',
+                                                         'early_stopping')
+        self._is_aedat_input = \
+            self.config.get('input', 'dataset_format') == 'aedat'
         self._poisson_input = self.config.getboolean('input', 'poisson_input')
         self._num_poisson_events_per_sample = \
             self.config.getint('input', 'num_poisson_events_per_sample')
@@ -512,10 +514,8 @@ class AbstractSNN:
 
         # Divide the test set into batches and run all samples in a batch in
         # parallel.
-        dataset_format = self.config.get('input', 'dataset_format')
-        num_batches = int(1e6) if dataset_format == 'aedat' else \
-            int(np.floor(self.config.getint('simulation', 'num_to_test') /
-                         self.batch_size))
+        num_batches = \
+            self.config.getint('simulation', 'num_to_test') // self.batch_size
 
         # Initialize intermediate variables for computing statistics.
         top5score_moving = 0
@@ -531,6 +531,9 @@ class AbstractSNN:
         path_acc = os.path.join(log_dir, 'accuracy.txt')
         if os.path.isfile(path_acc):
             os.remove(path_acc)
+        with open(path_acc, str('a')) as f_acc:
+            f_acc.write(str("# samples | SNN top-1 | top-{0} | ANN top-1 | "
+                            "top-{0}\n".format(self.top_k)))
 
         self.init_log_vars()
 
@@ -540,10 +543,10 @@ class AbstractSNN:
         data_batch_kwargs = {}
 
         # If DVS events are used as input, instantiate a DVSIterator.
-        if dataset_format == 'aedat':
+        if self._is_aedat_input:
             from snntoolbox.datasets.aedat.DVSIterator import DVSIterator
             batch_shape = list(np.array(
-                self.parsed_model.layers[0].batch_input_shape, int))
+                self.parsed_model.layers[0].input_shape, int).flatten())
             batch_shape[0] = self.batch_size
             # Get shape of input image, in case we need to subsample.
             image_shape = batch_shape[1:3] \
@@ -577,7 +580,7 @@ class AbstractSNN:
                     y_b_l = y_test[batch_idxs, :]
             elif dataflow is not None:
                 x_b_l, y_b_l = dataflow.next()
-            elif dataset_format == 'aedat':
+            elif self._is_aedat_input:
                 try:
                     data_batch_kwargs['dvs_gen'].next_sequence_batch()
                     y_b_l = data_batch_kwargs['dvs_gen'].y_b
@@ -587,6 +590,11 @@ class AbstractSNN:
                 # Generate frames so we can compare with ANN.
                 x_b_l = data_batch_kwargs['dvs_gen'].get_frame_batch()
 
+            # Effective batch size could be smaller than user-specified
+            # self.batch_size if dataset size not an integer multiple.
+            if len(x_b_l) < self.batch_size:
+                continue
+
             truth_b = np.argmax(y_b_l, axis=1)
 
             data_batch_kwargs['truth_b'] = truth_b
@@ -632,9 +640,6 @@ class AbstractSNN:
                 batch_idx + 1, num_batches, (batch_idx + 1) / num_batches))
             print("Moving accuracy of SNN (top-1, top-{}): {:.2%}, {:.2%}."
                   "".format(self.top_k, top1acc_moving, top5acc_moving))
-            with open(path_acc, str('a')) as f_acc:
-                f_acc.write(str("{} {:.2%} {:.2%}\n".format(
-                    num_samples_seen, top1acc_moving, top5acc_moving)))
 
             # Evaluate ANN on the same batch as SNN for a direct comparison.
             score = self.parsed_model.evaluate(
@@ -647,6 +652,11 @@ class AbstractSNN:
                   "\n".format(self.top_k, 1 - self.top1err_ann,
                               1 - self.top5err_ann))
 
+            with open(path_acc, str('a')) as f_acc:
+                f_acc.write(str("{} {:.2%} {:.2%} {:.2%} {:.2%}\n".format(
+                    num_samples_seen, top1acc_moving, top5acc_moving,
+                    1 - self.top1err_ann, 1 - self.top5err_ann)))
+
             # Plot input image.
             if 'input_image' in self._plot_keys:
                 snn_plt.plot_input_image(x_b_l[0], int(truth_b[0]), log_dir,
@@ -771,7 +781,7 @@ class AbstractSNN:
             layer_type = get_type(layer)
             if layer_type == 'Dense':
                 self.build_dense(layer)
-            elif layer_type in {'Conv2D', 'DepthwiseConv2D'}:
+            elif 'Conv' in layer_type:
                 self.build_convolution(layer)
                 self.data_format = layer.data_format
             elif layer_type in {'MaxPooling2D', 'AveragePooling2D'}:
@@ -1010,7 +1020,7 @@ class AbstractSNN:
         simulation.
         """
 
-        if self._poisson_input or self._dataset_format == 'aedat':
+        if self._poisson_input or self._is_aedat_input:
             spiketrains_b_l_t = self.get_spiketrains_input()
             if self.input_b_l_t is not None:
                 self.input_b_l_t = spiketrains_b_l_t
@@ -1790,3 +1800,29 @@ def remove_name_counter(name_in):
     # The first '_' is assigned by SNN toolbox and should be kept.
     return (split_underscore[0] + '_' + split_underscore[1] + '/' +
             split_dash[1])
+
+
+def convert_kernel(kernel):  # Copy of Keras code removed for deprecation
+    """Converts a Numpy kernel matrix from Theano format to TensorFlow format.
+
+    Also works reciprocally, since the transformation is its own inverse.
+
+    This is used for converting legacy Theano-saved model files.
+
+    Arguments:
+      kernel: Numpy array (3D, 4D or 5D).
+
+    Returns:
+      The converted kernel.
+
+    Raises:
+      ValueError: in case of invalid kernel shape or invalid data_format.
+    """
+
+    kernel = np.asarray(kernel)
+    if not 3 <= kernel.ndim <= 5:
+        raise ValueError('Invalid kernel shape:', kernel.shape)
+    slices = [slice(None, None, -1) for _ in range(kernel.ndim)]
+    no_flip = (slice(None, None), slice(None, None))
+    slices[-2:] = no_flip
+    return np.copy(kernel[slices])
