@@ -4,6 +4,8 @@ import inspect
 import numpy as np
 import os
 import shutil
+
+import pytest
 from tensorflow.keras import backend
 
 from snntoolbox.bin.utils import initialize_simulator, run_pipeline
@@ -12,16 +14,36 @@ from tests.conftest import pytorch_skip_if_dependency_missing
 from tests.core.test_models import get_correlations
 
 
+DATA_FORMAT = backend.image_data_format()
+
+
+def setup_module():
+    """Setup any state specific to the execution of pytorch tests."""
+
+    # Pytorch to Keras parser needs image_data_format == channel_first.
+    backend.set_image_data_format('channels_first')
+
+
+def teardown_module():
+    """Teardown any state that was previously setup with setup_module."""
+
+    # Reset to previous value.
+    backend.set_image_data_format(DATA_FORMAT)
+
+
 @pytorch_skip_if_dependency_missing
 class TestPytorchParser:
     """Test parsing pytorch models."""
 
-    @staticmethod
-    def prepare_model(model, config):
+    @pytest.fixture(scope='function', autouse=True)
+    def _setup(self, _config_first):
+        self.config = _config_first
+
+    def prepare_model(self, model):
         import torch
 
-        path_wd = config.get('paths', 'path_wd')
-        model_name = config.get('paths', 'filename_ann')
+        path_wd = self.config.get('paths', 'path_wd')
+        model_name = self.config.get('paths', 'filename_ann')
         torch.save(model.state_dict(), os.path.join(path_wd,
                                                     model_name + '.pkl'))
         # Need to copy model definition over to temp dir because only weights
@@ -30,12 +52,12 @@ class TestPytorchParser:
         source_path = inspect.getfile(pytorch)
         shutil.copyfile(source_path, os.path.join(path_wd, model_name + '.py'))
 
-    def test_loading(self, _model_4, _config):
+    def test_loading(self, _model_4_first):
 
         assert backend.image_data_format() == 'channels_first', \
             "Pytorch to Keras parser needs image_data_format == channel_first."
 
-        self.prepare_model(_model_4, _config)
+        self.prepare_model(_model_4_first)
 
         updates = {
             'tools': {'evaluate_ann': True,
@@ -47,29 +69,30 @@ class TestPytorchParser:
             'simulation': {'num_to_test': 100,
                            'batch_size': 50}}
 
-        _config.read_dict(updates)
+        self.config.read_dict(updates)
 
-        initialize_simulator(_config)
+        initialize_simulator(self.config)
 
-        normset, testset = get_dataset(_config)
+        normset, testset = get_dataset(self.config)
 
         model_lib = import_module('snntoolbox.parsing.model_libs.' +
-                                  _config.get('input', 'model_lib') +
+                                  self.config.get('input', 'model_lib') +
                                   '_input_lib')
-        input_model = model_lib.load(_config.get('paths', 'path_wd'),
-                                     _config.get('paths', 'filename_ann'))
+        input_model = model_lib.load(self.config.get('paths', 'path_wd'),
+                                     self.config.get('paths', 'filename_ann'))
 
         # Evaluate input model.
-        acc = model_lib.evaluate(input_model['val_fn'],
-                                 _config.getint('simulation', 'batch_size'),
-                                 _config.getint('simulation', 'num_to_test'),
-                                 **testset)
+        acc = model_lib.evaluate(
+            input_model['val_fn'],
+            self.config.getint('simulation', 'batch_size'),
+            self.config.getint('simulation', 'num_to_test'),
+            **testset)
 
         assert acc >= 0.8
 
-    def test_parsing(self, _model_4, _config):
+    def test_parsing(self, _model_4_first):
 
-        self.prepare_model(_model_4, _config)
+        self.prepare_model(_model_4_first)
 
         updates = {
             'tools': {'evaluate_ann': True,
@@ -82,17 +105,17 @@ class TestPytorchParser:
                            'batch_size': 50}
         }
 
-        _config.read_dict(updates)
+        self.config.read_dict(updates)
 
-        initialize_simulator(_config)
+        initialize_simulator(self.config)
 
-        acc = run_pipeline(_config)
+        acc = run_pipeline(self.config)
 
         assert acc[0] >= 0.8
 
-    def test_normalizing(self, _model_4, _config):
+    def test_normalizing(self, _model_4_first):
 
-        self.prepare_model(_model_4, _config)
+        self.prepare_model(_model_4_first)
 
         updates = {
             'tools': {'evaluate_ann': True,
@@ -105,17 +128,17 @@ class TestPytorchParser:
                            'batch_size': 50}
         }
 
-        _config.read_dict(updates)
+        self.config.read_dict(updates)
 
-        initialize_simulator(_config)
+        initialize_simulator(self.config)
 
-        acc = run_pipeline(_config)
+        acc = run_pipeline(self.config)
 
         assert acc[0] >= 0.8
 
-    def test_pipeline(self, _model_4, _config):
+    def test_pipeline(self, _model_4_first):
 
-        self.prepare_model(_model_4, _config)
+        self.prepare_model(_model_4_first)
 
         updates = {
             'tools': {'evaluate_ann': False},
@@ -127,14 +150,14 @@ class TestPytorchParser:
             'output': {
                 'log_vars': {'activations_n_b_l', 'spiketrains_n_b_l_t'}}}
 
-        _config.read_dict(updates)
+        self.config.read_dict(updates)
 
-        initialize_simulator(_config)
+        initialize_simulator(self.config)
 
-        acc = run_pipeline(_config)
+        acc = run_pipeline(self.config)
 
         assert acc[0] >= 0.8
 
-        corr = get_correlations(_config)
+        corr = get_correlations(self.config)
         assert np.all(corr[:-1] > 0.97)
         assert corr[-1] > 0.5
